/*
 *  Copyright (C) 2012 Thomas Luka, 18 rue des feuilles mortes, 95 Vaureal
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef ANDROID_OS
#define ANDROID_OS
#endif
#include "Fenetre.h"

MainWindow::MainWindow(QWidget *parent)
{  
  setWindowTitle(tr("Xcas"));

  m_HistCmd = new History;
#ifdef ANDROID_OS
  if (!QFile::exists("/sdcard/xcas/aide_cas"))
  {
      QDir dir;
      dir.setPath("/sdcard");
      dir.mkdir("xcas");
      QFile::copy("aide_cas", "/sdcard/xcas/aide_cas");
  }
  QFile file("/sdcard/xcas/session");
  if (file.open(QIODevice::ReadOnly | QIODevice::Text))
  {
        QStringList items;
        QTextStream in(&file);
        QString line = file.readLine();
        items.append(line);
        while (!in.atEnd())
        {
            line = in.readLine();
            items.append(line);
        }
        m_HistCmd->setAllItems(items);
  }
  m_Help = readhelp("/sdcard/xcas/aide_cas", m_IntHelp);
  sort(m_Help.begin(), m_Help.end(), alpha_order);
#endif

  m_Aide = new QAction("Aide", this);
  QToolButton *HelpButton = new QToolButton;
  HelpButton->setDefaultAction(m_Aide);//Pour que l'action aide soit lancer lorsque l'on clique sur le boutton

  QToolButton *LeftCursor = new QToolButton;
  LeftCursor->setText("<--");

  QToolButton *RightCursor = new QToolButton;
  RightCursor->setText("-->");

  QToolButton *About = new QToolButton;
  About->setText("?");

  QPushButton *clear_cmd_line = new QPushButton(tr("Effacer ligne"));

  QPushButton *stop = new QPushButton(tr("Stop"));
  stop->setEnabled(false);
#ifndef ANDROID_OS
  QMenu *Aide = menuBar()->addMenu(tr("&Aide"));
  Aide->addAction(m_Aide);
#endif

  m_Commande = new QLineEdit;

  QHBoxLayout *ToolLayout = new QHBoxLayout;
  ToolLayout->addWidget(LeftCursor);
  ToolLayout->setAlignment(LeftCursor, Qt::AlignLeft);
  ToolLayout->addWidget(About);
  ToolLayout->addWidget(HelpButton);
  ToolLayout->addWidget(RightCursor);
  ToolLayout->setAlignment(RightCursor, Qt::AlignRight);

  m_Layout = new QVBoxLayout;
  m_Layout->addLayout(ToolLayout);
  m_Layout->addWidget(m_Commande);
  m_Layout->addWidget(stop);
  m_Layout->addWidget(clear_cmd_line);
  m_Layout->addWidget(m_HistCmd);

  QWidget *Fen = new QWidget;
  Fen->setLayout(m_Layout);
  setCentralWidget(Fen);

  connect(m_Commande, SIGNAL(returnPressed()), this, SLOT(Evaluate()));
  connect(m_Aide, SIGNAL(triggered()), this, SLOT(ShowHelp()));
  connect(m_HistCmd, SIGNAL(strCpSignal(QString)), this, SLOT(addStrFromHist(QString)));
  connect(LeftCursor, SIGNAL(clicked()), this, SLOT(cursor_to_left()));
  connect(RightCursor, SIGNAL(clicked()), this, SLOT(cursor_to_right()));
  connect(About, SIGNAL(clicked()), this, SLOT(about_widget()));
  connect(clear_cmd_line, SIGNAL(clicked()), m_Commande, SLOT(clear()));
  connect(m_HistCmd, SIGNAL(calcul_en_cours(bool)), stop, SLOT(setEnabled(bool)));
  connect(stop, SIGNAL(clicked()), m_HistCmd, SLOT(stop_calcul()));
}
void MainWindow::Evaluate()
{
  QString Expression = qobject_cast<QLineEdit *>(sender())->text();
  m_HistCmd->addCmd(Expression);
  m_Commande->clear();
}
void MainWindow::ShowHelp()
{
  HelpWindow *Window;
#ifndef ANDROID_OS
  Window = new HelpWindow(m_Commande->text());
#endif
#ifdef ANDROID_OS
  Window = new HelpWindow(m_Help, m_IntHelp, m_Commande->text());
#endif
  connect(Window, SIGNAL(ExempleSelected(QString)), this, SLOT(writeExemple(QString)));
  connect(Window, SIGNAL(HelpClose()), this, SLOT(focusToLine()));
}
void MainWindow::writeExemple(QString ex)
{
  m_Commande->setText(ex);
}
void MainWindow::addStrFromHist(QString str)
{
  m_Commande->setText(m_Commande->text()+str);
}
void MainWindow::focusToLine()
{
    m_Commande->setFocus();
}
void MainWindow::cursor_to_left()
{
    m_Commande->setCursorPosition(m_Commande->cursorPosition()-1);
}
void MainWindow::cursor_to_right()
{
    m_Commande->setCursorPosition(m_Commande->cursorPosition()+1);
}
void MainWindow::about_widget()
{
    QString MessageAbout = "AndroCas est une GUI de giac";
    MessageAbout += QString::fromUtf8(" pour android développée par Thomas Luka");
    MessageAbout += QString::fromUtf8(" pour faire du calcul formel sur Androïd.");
    MessageAbout.insert(MessageAbout.size(), '\n');
    MessageAbout.insert(MessageAbout.size(), '\n');
    MessageAbout += QString::fromUtf8("Pour plus d'information sur giac veuillez voir à l'adresse http://www-fourier.ujf-grenoble.fr/~parisse/");
    QLabel *aboutLabel = new QLabel(MessageAbout);
    aboutLabel->setWordWrap(true);
    aboutLabel->setTextFormat(Qt::RichText);

    QPushButton *ok = new QPushButton("Ok");

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(aboutLabel);
    layout->addWidget(ok);

    QWidget *Fen = new QWidget;
    Fen->setLayout(layout);
    Fen->show();

    save_items();

    connect(ok, SIGNAL(clicked()), Fen, SLOT(close()));
}
void MainWindow::save_items()
{
    QStringList itemsHist = m_HistCmd->allCmds();
    QFile file("/sdcard/xcas/session");
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
             return;

    QTextStream out(&file);
    for (int i = 0; i < itemsHist.size(); i++)
        out << itemsHist.at(i) << "\n";
}
