/*
 *  Copyright (C) 2012 Thomas Luka, 18 rue des feuilles mortes, 95 Vaureal
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef ANDROID_OS
#define ANDROID_OS
#endif
#include "HelpWindow.h"

using namespace giac;

HelpWindow::HelpWindow(QString cmd)
{
#ifndef ANDROID_OS
  Language();
  m_ExempleSelec = "";//On initialise la chaine des exemples

  m_CasHelp = readhelp("/sdcard/aide_cas",m_CountHelp);//On recupere les commandes
  sort(m_CasHelp.begin(), m_CasHelp.end(), alpha_order);//On trie les commandes par ordre alphabetique
  QStringList ListesCommandes = allHelpCmd(m_CasHelp);//Pour obtenir les commandes

  m_CommandesModel = new QStandardItemModel;//On dclare le model
  for (int i = 0; i != ListesCommandes.size(); i++)
    {
      QStandardItem *ItemCmd = new QStandardItem(ListesCommandes.at(i));
      m_CommandesModel->appendRow(ItemCmd);//On ajoute les commandes
    }
  m_CommandesView = new QListView;
  m_CommandesView->setModel(m_CommandesModel);
  m_CommandesView->setEditTriggers(QAbstractItemView::NoEditTriggers);

  /*GroupBox Pour les informations des commandes + Rechercher*/
  m_Description = new QGroupBox(tr("Description : "));
  m_DescriptionLabel = new QLabel;
  m_DescriptionLabel->setWordWrap(true);
  QVBoxLayout * DescriptionLayout = new QVBoxLayout;
  DescriptionLayout->addWidget(m_DescriptionLabel);
  m_Description->setLayout(DescriptionLayout);

  m_Exemples = new QGroupBox(tr("Exemples : "));
  m_LayoutExemple = new QVBoxLayout;
  m_Exemples->setLayout(m_LayoutExemple);


  m_CommandesSimilaires = new QGroupBox(tr("Voir aussi : "));
  m_LayoutSimilaire = new QGridLayout;
  m_CommandesSimilaires->setLayout(m_LayoutSimilaire);

  m_rechercher = new QLineEdit;

  /*Configuration du layout qui contient les GroupBox et la line edit pour la recherche
    Sous android aussi la vue */
  QVBoxLayout * GroupBoxLayout = new QVBoxLayout;
  GroupBoxLayout->addWidget(m_rechercher);
  GroupBoxLayout->addWidget(m_CommandesView);

  GroupBoxLayout->addWidget(m_Description);
  GroupBoxLayout->addWidget(m_Exemples);
  GroupBoxLayout->addWidget(m_CommandesSimilaires);
  QPushButton *BouttonOk = new QPushButton("Ok");
  QPushButton *BouttonAnnuler = new QPushButton("Annuler");

  /*Layout pour les bouttons*/
  QHBoxLayout *BouttonsLayout = new QHBoxLayout;
  BouttonsLayout->addWidget(BouttonOk);
  BouttonsLayout->addWidget(BouttonAnnuler);
  BouttonsLayout->setAlignment(Qt::AlignRight);

  BouttonsLayout->setAlignment(Qt::AlignLeft);
  GroupBoxLayout->addLayout(BouttonsLayout);

  /*Layout pour avoir la vue et les groupBoxs*/
  QHBoxLayout * VueLayout = new QHBoxLayout;
  VueLayout->addWidget(m_CommandesView);
  VueLayout->addLayout(GroupBoxLayout);

  /*Layout de la fenetre*/
  QVBoxLayout *FenetreLayout = new QVBoxLayout;
  FenetreLayout->addLayout(VueLayout);
  FenetreLayout->addLayout(BouttonsLayout);
  setLayout(FenetreLayout);

  setWindowTitle(tr("Aide Cas"));

  connect(BouttonAnnuler, SIGNAL(clicked()), this, SLOT(close()));
  connect(m_CommandesView, SIGNAL(clicked (const QModelIndex)), this, SLOT(itemClicked(const QModelIndex)));
  connect(BouttonOk, SIGNAL(clicked()), this, SLOT(OkClicked()));
  connect(m_rechercher, SIGNAL(returnPressed()), this, SLOT(LineEditCmd()));
  connect(m_rechercher, SIGNAL(textChanged(const QString)), this, SLOT(LineEditModified(const QString)));

  show();
#endif //android ndef

#ifdef ANDROID_OS

  Language();
  CreateAndroidWin();

#endif //android def
  m_rechercher->setText(cmd);
}
HelpWindow::HelpWindow(std::vector<aide> &helpVector, int CountHelp, QString cmd)
{
    Language();
    CreateAndroidWin();//Toutes les initialisation de la fenetre
    m_CasHelp = helpVector;
    m_CountHelp = CountHelp;
    for(unsigned int i = 1; i != m_CasHelp.size(); i++)
      {
        QStandardItem * Item = new QStandardItem(QString::fromStdString(m_CasHelp[i].cmd_name));
        m_CommandesModel->appendRow(Item);
      }
    m_rechercher->setText(cmd);
}

void HelpWindow::itemClicked(const QModelIndex & index)
{
    m_rechercher->setText(m_CommandesModel->data(index, Qt::DisplayRole).toString());
#ifndef ANDROID_OS
  CompleterChamp(m_CommandesModel->data(index, Qt::DisplayRole).toString());//On recupere le nom de la commande
#endif
}
void HelpWindow::Language()
{
#ifdef ANDROID_OS
    m_language = 1;
    return;
#endif
  if ( QLocale::system().language() == QLocale::French )
    {
      m_language = 1;
    }
  if ( QLocale::system().language() == QLocale::C )
    {
      m_language = 2;
    }
  if ( QLocale::system().language() == QLocale::Spanish )
    {
      m_language = 3;
    }
  if ( QLocale::system().language() == QLocale::Greek )
    {
      m_language = 4;
    }
  if ( QLocale::system().language() == QLocale::Chinese )
    {
      m_language = 8;
    }
}
void HelpWindow::RadioBouttonToggle(bool Checked)
{
  if (Checked)
    {
      m_ExempleSelec = qobject_cast<QRadioButton *>(sender())->text();
      if (m_ExempleSelec.contains('\n'))
      {
          m_ExempleSelec.remove(QChar('\n'));
      }
    }
}
aide HelpWindow::RechercherCommande(QString Commande)
{
  aide Cmd;
  Cmd.cmd_name = Commande.toStdString();

  unsigned int i = m_CasHelp.size()/2;//On definit i pou qu'il soit la moiti
  int begin = 0;//Le debut pour les recherche
  int end = m_CasHelp.size();//la fin  pour les recherches
  while(i != m_CasHelp.size())
    {
      if (Commande == QString::fromStdString(m_CasHelp.at(i).cmd_name))
        {
	  return m_CasHelp.at(i);
        }
      if (alpha_order(Cmd, m_CasHelp.at(i)))
        {
	  end = i;
	  i = (end - begin)/2;
	  i += begin;
        }
      else
        {
	  begin = i;
	  i += (end-i)/2;
        }
    }
  aide NotFound;
  NotFound.cmd_name = "";
  return NotFound;
}
QString HelpWindow::DescriptionCommande(const aide & cmd)
{
  std::vector<localized_string> VectorDescription = cmd.blabla;
  QString Description;

  /*Pour la description dans la bonne langue*/
  for (unsigned int i = 0; VectorDescription.size() != i; i++)
    {
      if ( VectorDescription.at(i).language == m_language )
        {
	  Description = QString::fromUtf8(VectorDescription.at(i).chaine.data());
	  break;
        }
    }

  /*Pour avoir la syntaxe*/
  Description.insert(Description.size(), '\n');
  Description.insert(Description.size(), '\n');
  Description += "Syntaxe : ";
  Description += QString::fromStdString(cmd.syntax);

  return Description;
}
QStringList HelpWindow::ExemplesCommande(const aide & cmd)
{
  QStringList Exemples;
  for(unsigned int i = 0; i != cmd.examples.size(); i++)
    {
      QString Chaine = QString::fromStdString(cmd.examples.at(i));//On recupere l'exemple
#ifdef ANDROID_OS
      for (int i = 20; Chaine.size() > i; i+=20)
      {
          Chaine.insert(i, '\n');
      }
#endif
      Exemples.append(Chaine);
    }
  return Exemples;
}
QStringList HelpWindow::Similitudes(const aide & cmd, bool syn)
{
  QStringList Simile;
  for (unsigned int i = 0; i != cmd.related.size(); i++)
    {
      Simile.append(QString::fromStdString(cmd.related.at(i).chaine));
    }
  if (syn)
    {
      for (unsigned int i = 0; i != cmd.synonymes.size(); i++)
        {
	  if (cmd.synonymes.at(i).language == m_language)
            {
	      Simile.append(QString::fromStdString(cmd.synonymes.at(i).chaine));
            }
        }
    }
  return Simile;
}
void HelpWindow::OkClicked()
{
    if (!m_ExempleSelec.isEmpty())//Si la chaine est vide on laisse la chaine d'origine dans la QLineEdit
        emit ExempleSelected(m_ExempleSelec);

    emit HelpClose();
    close();
}
void HelpWindow::CmdClicked()
{
#ifndef ANDROID_OS
  CompleterChamp(qobject_cast<QPushButton *>(sender())->text());
#endif
  m_rechercher->setText(qobject_cast<QPushButton *>(sender())->text());
}
void HelpWindow::CompleterChamp(QString cmd, ChampCompleter valueCh)
{
  aide CommandeClicked = RechercherCommande(cmd);//On recupere l'aide de la commande

  QList<QStandardItem *> ItemCommande = m_CommandesModel->findItems(cmd, Qt::MatchExactly);//On recupere les items des commandes
  /*Gestion des selections et du currentindex pour la convivialite*/
  if (!ItemCommande.isEmpty())
      m_CommandesView->setCurrentIndex(ItemCommande.at(0)->index());//Pour Modifier l'index courrant

  /*Description*/
  if (valueCh == ALL || valueCh == DESCRIPTION)
         m_DescriptionLabel->setText(DescriptionCommande(CommandeClicked));

  /*Exemples*/
  if (valueCh == ALL || valueCh == EXEMPLES)
  {
      m_LayoutExemple->setParent(this->layout());
      /*On efface les Exemples de la commande precedente qui ne doivent plus etre dans le layout*/
      QLayoutItem *child;
      while ((child = m_LayoutExemple->takeAt(0)) != 0)
        {
          child->widget()->deleteLater();
          delete child;
        }
      QRadioButton *Commande = new QRadioButton(cmd);//On ajoute la commande comme possibilite
      m_LayoutExemple->addWidget(Commande);
      connect(Commande, SIGNAL(toggled(bool)), this, SLOT(RadioBouttonToggle(bool)));
      /*On ajoute tous les autres exemples*/
      QStringList ExemplesCmd = ExemplesCommande(CommandeClicked);
      for(int i = 0; i != ExemplesCmd.size(); i++)
        {
          if ( i == 4)
              break;

          QRadioButton * Button = new QRadioButton(ExemplesCmd.at(i));
          m_LayoutExemple->addWidget(Button);//On ajoute le RadioBoutton au Layout
          connect(Button, SIGNAL(toggled(bool)), this, SLOT(RadioBouttonToggle(bool)));
        }
    }

  /*Commandes simmilaires et synonyms*/
  if (valueCh == ALL || valueCh == VOIRAUSSI)
  {
      m_LayoutSimilaire->setParent(this->layout());
      /*Pour effacer les eventuelles widget restant*/
      QLayoutItem *child;
      while ((child = m_LayoutSimilaire->takeAt(0)) != 0)//On reprend un QlayoutItem child comme precedamment
        {
          child->widget()->deleteLater();
          delete child;
        }
      delete child;
      int lineLayout = 0;//Pour GridLayout
      int ColonneLayout = 0;//Pour GridLayout
      int ColonneLimite = 5;//Pour definir le nombre de commande dans la layout par ligne
    #ifdef ANDROID_OS
      ColonneLimite = 1;//Pour eviter une surcharge en largeur qui ferait dépasser la fenetre de l'ecran
    #endif
      QStringList CmdSimil = Similitudes(CommandeClicked, true);
      for (int i = 0; i != CmdSimil.size(); i++)
        {
          if (!CmdSimil.at(i).isEmpty())
            {
              QPushButton *BouttonCommande = new QPushButton(CmdSimil.at(i));
              m_LayoutSimilaire->addWidget(BouttonCommande, lineLayout, ColonneLayout);
              connect(BouttonCommande, SIGNAL(clicked()), this, SLOT(CmdClicked()));
              /*connect for android only, ifndef ANDROID_OS this connect is no effects*/
              connect(BouttonCommande, SIGNAL(clicked()), this, SLOT(VoirAussiClicked()));

              ColonneLayout++;
              /*Pour eviter que la ligne soit trop grande on utilise un GridLayout et on va a la ligne*/
              if (ColonneLayout == ColonneLimite)
                {
                  ColonneLayout = 0;
                  lineLayout++;
                }
            }
        }
  }
}
void HelpWindow::LineEditCmd()
{
  QString CmdLine = qobject_cast<QLineEdit *>(sender())->text();
  if (m_CommandesModel->findItems(CmdLine).isEmpty())
    {
      /*Creation de la QDialog*/
      QDialog *Dialog = new QDialog;
      Dialog->setWindowTitle("Erreur de recherche");

      /*On recupere les commandes qui peuvent tre simmilaires (related)*/
      std::vector<indexed_string> Related= helpon(CmdLine.toStdString(), m_CasHelp, m_language, m_CasHelp.size(), true).related;//On recheche les commandes simmilaires

      /*On cree la chaine de caractere qui s'affichera dans la fenetre*/
      QString Message = QString(tr("La commande "))+CmdLine+QString(tr(" n'existe pas"));//On cree notre chaine message
      Message.insert(Message.size(), '\n');
      Message += "Veuillez resaisir la commande.";
      Message.insert(Message.size(), '\n');
      if (Related.size() != 0)
        Message += QString::fromUtf8("Peut-être vouliez-vous dire : ");

      QLabel *MessageLabel = new QLabel(Message);
      MessageLabel->setWordWrap(true);
      QVBoxLayout *LabelLayout = new QVBoxLayout;
      LabelLayout->addWidget(MessageLabel);

      QGridLayout *LayoutCmd = new QGridLayout;
      int ligne = 0;
      int colonne = 0;
      int colonneLimit = 4;
      int ligneLimit = 100;//Pour eviter une surcharge de ligne surtout pour android
#ifdef ANDROID_OS
      colonneLimit = 1;//Pour eviter une surchage dans la largeur et donc un pb d'affichage
      ligneLimit = 7;//Même raison mais en hauteru donc ecrasement des bouttons
#endif
      for (unsigned int i = 0; i != Related.size(); i++)//Pour ne pas ajouter trop de bouttons
        {
          if (colonne == colonneLimit)
            {
	      colonne = 0;
	      ligne++;
            }
          if (ligne == ligneLimit)
              break;

	  QPushButton *BouttonCmd = new QPushButton(QString::fromStdString(Related.at(i).chaine));//On ajoute un boutton avec le text de la commande
	  LayoutCmd->addWidget(BouttonCmd, ligne, colonne);
	  connect(BouttonCmd, SIGNAL(clicked()), this, SLOT(CmdClicked()));
	  connect(BouttonCmd, SIGNAL(clicked()), Dialog, SLOT(close()));//Pour fermer la fenetre
	  colonne++;
        }
      QVBoxLayout *OkLayout = new QVBoxLayout;
      QPushButton *OkButton = new QPushButton(tr("Ok"));
      OkLayout->addWidget(OkButton);
      OkLayout->setAlignment(Qt::AlignLeft);

      QVBoxLayout *DialogLayout = new QVBoxLayout;
      DialogLayout->addLayout(LabelLayout);
      DialogLayout->addLayout(LayoutCmd);
      DialogLayout->addLayout(OkLayout);

      Dialog->setLayout(DialogLayout);
      Dialog->showFullScreen();

      connect(OkButton, SIGNAL(clicked()), Dialog, SLOT(close()));
    }
  else
    {
#ifndef ANDROID_OS
      CompleterChamp(CmdLine);
#endif
    }
}
QStringList HelpWindow::allHelpCmd(std::vector<aide> & VectorCmd)
{
  QStringList cmds;
  for(unsigned int i = 0; i != VectorCmd.size(); i++)
    {
      cmds.append(QString::fromStdString(VectorCmd.at(i).cmd_name));
    }
  return cmds;
}
void HelpWindow::LineEditModified(const QString NewText)
{
  QList<QStandardItem *> Items = m_CommandesModel->findItems(NewText, Qt::MatchCaseSensitive);
  if (Items.isEmpty())
      Items = m_CommandesModel->findItems(NewText, Qt::MatchStartsWith);//Item qui commence par l'expression pour pouvoir les selectionner
  if (!Items.isEmpty())
      m_CommandesView->setCurrentIndex(Items.at(0)->index());//Pour Modifier l'index courrant
}
void HelpWindow::DescriptionWindow()
{
    CompleterChamp(m_CommandesModel->itemFromIndex(m_CommandesView->currentIndex())->text(), DESCRIPTION);

    QPushButton *ok = new QPushButton("Ok");

    QVBoxLayout *Win = new QVBoxLayout;
    Win->addWidget(m_DescriptionLabel);
    Win->addWidget(ok);

    QWidget *desFen = new QWidget;
    desFen->setLayout(Win);
    desFen->showMaximized();

    connect(ok, SIGNAL(clicked()), desFen, SLOT(close()));
}
void HelpWindow::ExemplesWindow()
{
    CompleterChamp(m_CommandesModel->itemFromIndex(m_CommandesView->currentIndex())->text(), EXEMPLES);

    QPushButton *ok = new QPushButton(tr("Copier l'exemple"));
    QPushButton *annul = new QPushButton(tr("Annuler"));

    QVBoxLayout *winLayout = new QVBoxLayout;
    winLayout->addLayout(m_LayoutExemple);
    winLayout->addWidget(ok);
    winLayout->addWidget(annul);

    QWidget *exWin = new QWidget();
    exWin->setLayout(winLayout);
    exWin->showMaximized();

    connect(ok, SIGNAL(clicked()), this, SLOT(OkClicked()));
    connect(ok, SIGNAL(clicked()), exWin, SLOT(close()));
    connect(ok, SIGNAL(clicked()), this, SLOT(AnnulExempleClicked()));
    connect(annul, SIGNAL(clicked()), exWin, SLOT(close()));
}
void HelpWindow::VoirAussiWindow()
{
    CompleterChamp(m_CommandesModel->itemFromIndex(m_CommandesView->currentIndex())->text(), VOIRAUSSI);

    QString textLabel = QString::fromUtf8("Vous pouvez également voir les commandes suivantes : ");
    QLabel *label = new QLabel;
    label->setText(textLabel);
    label->setWordWrap(true);
    QPushButton *ok = new QPushButton("Ok");

    QVBoxLayout *vaLayout = new QVBoxLayout;
    vaLayout->addWidget(label);
    vaLayout->addLayout(m_LayoutSimilaire);
    vaLayout->addWidget(ok);

    QWidget *vaWid = new QWidget;
    vaWid->setLayout(vaLayout);
    vaWid->showMaximized();

    connect(ok, SIGNAL(clicked()), vaWid, SLOT(close()));
}
void HelpWindow::VoirAussiClicked()
{
#ifdef ANDROID_OS
    QWidget *vaWid = qobject_cast<QPushButton *>(sender())->parentWidget();
    m_LayoutSimilaire->setParent(this);
    vaWid->close();
#endif
}
void HelpWindow::CreateAndroidWin()
{
    m_CommandesModel = new QStandardItemModel;//On declare le modele
    m_CommandesView = new QListView;
    m_CommandesView->setModel(m_CommandesModel);
    m_CommandesView->setEditTriggers(QAbstractItemView::NoEditTriggers);

    m_DescriptionLabel = new QLabel;
    m_DescriptionLabel->setWordWrap(true);
    m_LayoutExemple = new QVBoxLayout;
    m_LayoutSimilaire = new QGridLayout;

    m_rechercher = new QLineEdit;
    QPushButton *BouttonOk = new QPushButton("Ok");

    QPushButton *BouttonDescription = new QPushButton(tr("Description"));
    QPushButton *BouttonExemples = new QPushButton(tr("Exemples"));
    QPushButton *BouttonVoirAussi = new QPushButton(tr("Voir aussi"));

    /*Configuration du layout qui contient les GroupBox et la line edit pour la recherche
      Sous android aussi la vue et le boutton ok*/
    QVBoxLayout * HelpLayout = new QVBoxLayout;
    HelpLayout->addWidget(m_rechercher);
    HelpLayout->addWidget(m_CommandesView);
    HelpLayout->addWidget(BouttonDescription);
    HelpLayout->addWidget(BouttonExemples);
    HelpLayout->addWidget(BouttonVoirAussi);
    HelpLayout->addWidget(BouttonOk);
    setLayout(HelpLayout);

    setWindowTitle(tr("Aide Cas"));

    showMaximized();

    connect(m_CommandesView, SIGNAL(clicked (const QModelIndex)), this, SLOT(itemClicked(const QModelIndex)));
    connect(BouttonOk, SIGNAL(clicked()), this, SLOT(OkClicked()));
    connect(m_rechercher, SIGNAL(returnPressed()), this, SLOT(LineEditCmd()));
    connect(m_rechercher, SIGNAL(textChanged(const QString)), this, SLOT(LineEditModified(const QString)));
    connect(BouttonDescription, SIGNAL(clicked()), this, SLOT(DescriptionWindow()));
    connect(BouttonExemples, SIGNAL(clicked()), this, SLOT(ExemplesWindow()));
    connect(BouttonVoirAussi, SIGNAL(clicked()), this, SLOT(VoirAussiWindow()));
}
void HelpWindow::AnnulExempleClicked()
{
    m_ExempleSelec.clear();//Pour que la chaine soit vide est que le texte de la cmdLine ne soit pas effacer a la fermeytre de l'aide
}
